package palmbible.output;

import java.io.*;
import java.util.*;

import palmbible.parser.*;
import palmbible.util.*;
import palmbible.data.*;

public abstract class GenericDoc
{
	protected BibleDoc bibleDoc;
	protected String docName;

	protected String templatePackage;
	
	protected File outputDir;
	protected File outputSubDir;
	protected Map writerMap = new HashMap();
	protected Map printListMap = new HashMap();
	protected boolean store = true;
	
	protected String encode;

	protected static final int[] bookTable = {
		10, 20, 30, 40, 50, 
		60, 70, 80, 90, 100, 
		110, 120, 130, 140, 150, 
		160, 190, 220, 230, 240, 
		250, 260, 290, 300, 310, 
		330, 340, 350, 360, 370, 
		380, 390, 400, 410, 420, 
		430, 440, 450, 460, 470, 
		480, 490, 500, 510, 520, 
		530, 540, 550, 560, 570, 
		580, 590, 600, 610, 620, 
		630, 640, 650, 660, 670, 
		680, 690, 700, 710, 720, 
		730
		, 170, 180, 200, 210, 270, 280, 320
	};

	protected static final String[] bookEngName = {
		"Genesis", "Exodus", "Leviticus", "Numbers", "Deuteronomy",
		"Joshua", "Judges", "Ruth", "1 Samuel", "2 Samuel",
		"1 Kings", "2 Kings", "1 Chronicles", "2 Chronicles", "Ezra",
		"Nehemiah", "Esther", "Job", "Psalms", "Proverbs",
		"Ecclesiastes", "Song of Solomon", "Isaiah", "Jeremiah", "Lamentations",
		"Ezekiel", "Daniel", "Hosea", "Joel", "Amos",
		"Obadiah", "Jonah", "Micah", "Nahum", "Habakkuk",
		"Zephaniah", "Haggai", "Zechariah", "Malachi", "Matthew",
		"Mark", "Luke", "John", "Acts", "Romans",
		"1 Corinthians", "2 Corinthians", "Galatians", "Ephesians", "Philippians",
		"Colossians", "1 Thessalonians", "2 Thessalonians", "1 Timothy", "2 Timothy",
		"Titus", "Philemon", "Hebrews", "James", "1 Peter",
		"2 Peter", "1 John", "2 John", "3 John", "Jude",
		"Revelation", 
		"Tobit", "Judith", "1 Maccabees", "2 Maccabees",
		"Wisdom", "Sirach", "Baruch"
	};

	public GenericDoc(BibleDoc bibleDoc, String docName) throws Exception
	{
		this.bibleDoc = bibleDoc;
		this.docName = docName;
	}

	public void generate() throws Exception
	{
		WordRecorder wordRecorder = bibleDoc.getWordRecorder();
		boolean breakUnicode = bibleDoc.getBreakUnicode();

		Map wordMap = new HashMap();

		wordMap.put("%%converter%%", Info.version);

		wordMap.put("%%docname%%", docName);
		wordMap.put("%%version%%", bibleDoc.getBibleDocName());
		wordMap.put("%%info%%", bibleDoc.getBibleDocInfo());
		
		if (encode == null){
			encode = bibleDoc.getEncoding();
		}
		wordMap.put("%%encode%%", encode);

		store = true;

		for (int i=0; i < bibleDoc.getTotalBook(); i++){
			Book book = bibleDoc.getBook(i);
			wordMap.put("%%bookname%%", book.getBookName());
			wordMap.put("%%bookshortcut%%", book.getBookShortcut());
			
			int bookNum = book.getBookNumber();
			wordMap.put("%%booknum%%", Integer.toString(bookNum));
			wordMap.put("%%booknumtext%%", convertNum(bookNum));

			int bookSeq = getBookSeq(bookNum);
			wordMap.put("%%bookseq%%", Integer.toString(bookSeq));
			wordMap.put("%%bookseqtext%%", convertNum(bookSeq));
			
			if (bookSeq > 0){
				wordMap.put("%%bookengname%%", bookEngName[bookSeq - 1]);
			} else {
				wordMap.put("%%bookengname%%", "UNKNOWN");
			}

			for (int j=0; j < book.getTotalChapter(); j++){
				Chapter chapter = book.getChapter(j);

				int chapNum = j + 1;
				wordMap.put("%%chapnum%%", Integer.toString(chapNum));
				wordMap.put("%%chapnumtext%%", convertNum(chapNum));

				for (int k=0; k < chapter.getTotalVerse(); k++){
					Verse verse = chapter.getVerse(k);
					
					int versNum = k + 1;
					wordMap.put("%%versnum%%", Integer.toString(versNum));
					wordMap.put("%%versnumtext%%", convertNum(versNum));

					String content = verse.getString();
					StringBuffer buffer = new StringBuffer();

					int lastType = -1;
					int type = 0xFFFC;
					
					int lastChar = 0xFFFC;

					for (int m=0; m < verse.getTotalChar(); m++){
						int ch = content.charAt(m);
						
						switch (ch){
							case 0xFFFF:
							case 0xFFFE:
							case 0xFFFD:
							case 0xFFFC:
								type = ch;
								if (lastType != -1){
									outputText(lastType, wordMap, buffer.toString());
									buffer = new StringBuffer();
								}
								lastType = -1;
								break;
							default:
								String word = wordRecorder.getWordString(ch);

								if (!breakUnicode){
									if (word.length() == 1){
										int wordChar = word.charAt(0);
	
										switch (wordChar){
											case '?': case '!':	case ':': case ';':
											case '.': case ',': case '\'':
												break;
											default :
												if (lastType != -1){
													buffer.append(' ');
												}
										}
 									} else {
										if (lastType != -1){
											buffer.append(' ');
										}
									}									
								}

								buffer.append(word);
								lastType = type;
						}

						lastChar = ch;
					}

					outputText(type, wordMap, buffer.toString());

					store = false;
					
					if (i ==0 && j == 0 && k == 0) {
						beginBible(wordMap);
					}

					if (j == 0 && k == 0) {
						beginBook(wordMap);
					}

					if (k == 0) {
						beginChapter(wordMap);
					}
					
					beginVerse(wordMap);
					printStored();
					endVerse(wordMap);
					
					store = true;
				}
				endChapter(wordMap);
				printStored();
			}
			endBook(wordMap);
			printStored();
		}
		endBible(wordMap);
		printStored();

		Set keySet = printListMap.keySet();
		Iterator keys = keySet.iterator();

		while (keys.hasNext()){
			PrintWriter writer = getWriter((String) keys.next());
			writer.close();
		}
	}

	private void outputText(int type, Map wordMap, String text) throws Exception
	{
		if (!text.trim().equals("")){
			switch (type){
				case 0xFFFF:
					wordMap.put("%%booktext%%", text);
					bookText(wordMap);
					break;
				case 0xFFFE:
					wordMap.put("%%chaptext%%", text);
					chapterText(wordMap);
					break;
				case 0xFFFD:
					wordMap.put("%%desctext%%", text);
					titleText(wordMap);
					break;
				case 0xFFFC:
					wordMap.put("%%verstext%%", text);
					verseText(wordMap);
					break;
			}
		}
	}

	protected void setTemplatePackage(String packageName) throws Exception
	{
		templatePackage = packageName;
	}
	
	protected Template getTemplate(String templateName) throws Exception
	{
		return new Template(templatePackage + templateName);
	}
	
	protected void setOutputDir(String dirName) throws Exception
	{
		outputDir = new File(dirName);
		outputDir.mkdirs();
	}
	
	protected void setOutputSubDir(String dirName) throws Exception
	{
		outputSubDir = new File(outputDir, dirName);
		outputSubDir.mkdirs();
	}

	protected void setOutputFile(String fileName) throws Exception
	{
		setOutputFile(fileName, "default");
	}

	protected void setOutputFile(String fileName, String type) throws Exception
	{
		File outputFile;

		if (outputSubDir == null){
			outputFile = new File(outputDir, fileName);
		} else {
			outputFile = new File(outputSubDir, fileName);
		}
		
		if (outputFile.exists()){
			outputFile.delete();
		}

		PrintWriter writer = getWriter(type);

		if (writer != null){
			writer.close();
		}

		writer = new PrintWriter(
			new BufferedWriter(
			new OutputStreamWriter(
			new FileOutputStream(outputFile), encode)));

		writerMap.put(type, writer);
	}

	private PrintWriter getWriter(String type)
	{
		return (PrintWriter) writerMap.get(type);
	}
	
	private Vector getPrintList(String type)
	{
		Vector printList = (Vector) printListMap.get(type);
		
		if (printList == null) {
			printList = new Vector();
			printListMap.put(type, printList);
		}

		return printList;
	}

	private void printOut(String text, String type)
	{
		PrintWriter writer = getWriter(type);
		writer.print(text);
	}

	protected void print(String text) throws Exception
	{
		print(text, "default");
	}
	
	protected void print(String text, String type) throws Exception
	{
		if (store){
			Vector printList = getPrintList(type);
			printList.add(text);
		} else {
			printOut(text, type);
		}
	}

	protected void printStored() throws Exception
	{
		Set keySet = printListMap.keySet();
		Iterator keys = keySet.iterator();

		while (keys.hasNext()){
			printStored((String) keys.next());
		}
	}
	
	protected void printStored(String type) throws Exception
	{
		Vector printList = getPrintList(type);

		for(int i=0; i< printList.size(); i++){
			printOut((String) printList.elementAt(i), type);
		}
		
		clearStored(type);
	}

	protected void clearStored() throws Exception
	{
		Set keySet = printListMap.keySet();
		Iterator keys = keySet.iterator();

		while (keys.hasNext()){
			clearStored((String) keys.next());
		}
	}
	
	protected void clearStored(String type) throws Exception
	{
		Vector printList = getPrintList(type);
		printList.clear();
	}
	
	protected int getBookSeq(int bookNum)
	{
		for (int i = 0; i < bookTable.length; i++){
			if (bookNum == bookTable[i]){
				return i + 1;
			}
		}
		
		return 0;
	}
	
	protected String convertNum(int number) throws Exception
	{
		String numText = Integer.toString(number);

		if (number < 10){
			numText = "0" + numText;
		}

		if (number < 100){
			numText = "0" + numText;
		}
		
		return numText;
	}

	protected void outputFile(String fileName, Map wordMap) throws Exception
	{
		outputFile(fileName, null, wordMap);
	}

	protected void outputFile(String fileName, String dirName, Map wordMap) throws Exception
	{
		File dir = outputDir;
	
		if (dirName != null){
			dir = new File(outputDir, dirName);
		}

		File file = new File(dir, fileName);
		Template template = getTemplate(fileName);

		PrintWriter writer = new PrintWriter(
			new BufferedWriter(
			new OutputStreamWriter(
			new FileOutputStream(file), encode)));

		writer.print(template.getString(wordMap));
		writer.close();
	}

	protected void beginBible(Map wordMap) throws Exception
	{
	}
	
	protected void endBible(Map wordMap) throws Exception
	{
	}
	
	protected void beginBook(Map wordMap) throws Exception
	{
	}
	
	protected void endBook(Map wordMap) throws Exception
	{
	}
	
	protected void beginChapter(Map wordMap) throws Exception
	{
	}
	
	protected void endChapter(Map wordMap) throws Exception
	{
	}
	
	protected void beginVerse(Map wordMap) throws Exception
	{
	}
	
	protected void endVerse(Map wordMap) throws Exception
	{
	}
	
	protected void bookText(Map wordMap) throws Exception
	{
	}
	
	protected void verseText(Map wordMap) throws Exception
	{
	}
	
	protected void chapterText(Map wordMap) throws Exception
	{
	}
	
	protected void titleText(Map wordMap) throws Exception
	{
	}
}
