package palmbible.output;

import java.io.*;
import java.util.*;

import palmbible.parser.*;
import palmbible.pdbfile.*;
import palmbible.compress.*;
import palmbible.util.*;

public class HTMLDoc extends GenericDoc
{
	protected Template headTP;
	protected Template footTP;
	protected Template blankTP;
	protected Template bookTP;
	protected Template chapterTP;
	protected Template titleTP;
	protected Template verseTP;
	protected Template pageTP;

	protected Template tocHeadTP;
	protected Template tocFootTP;
	protected Template tocBookHeadTP;
	protected Template tocBookFootTP;
	protected Template tocChapterTP;

	protected Template chapterTocHeadTP;
	protected Template chapterTocFootTP;
	protected Template chapterTocChapterTP;
	protected Template chapterTocChapterHeadTP;
	protected Template chapterTocChapterFootTP;

	protected Template bookTocHeadTP;
	protected Template bookTocFootTP;
	protected Template bookTocBookTP;

	protected boolean lastBlank = true;
	protected int chapterCount = 0;

	public HTMLDoc(BibleDoc bibleDoc, String docName) throws Exception
	{
		super(bibleDoc, docName);
	
		setTemplatePackage("/template/html/");

		headTP 		= getTemplate("html.head");
		footTP 		= getTemplate("html.foot");
		bookTP 		= getTemplate("html.book");
		chapterTP 	= getTemplate("html.chapter");
		titleTP 	= getTemplate("html.title");
		verseTP 	= getTemplate("html.verse");
		blankTP 	= getTemplate("html.blank");

		tocHeadTP 		= getTemplate("toc.hhc.head");
		tocFootTP 		= getTemplate("toc.hhc.foot");
		tocBookHeadTP 	= getTemplate("toc.hhc.book.head");
		tocBookFootTP 	= getTemplate("toc.hhc.book.foot");
		tocChapterTP 	= getTemplate("toc.hhc.chapter");
		
		chapterTocHeadTP 		= getTemplate("chapter.toc.html.head");
		chapterTocFootTP 		= getTemplate("chapter.toc.html.foot");
		chapterTocChapterTP 	= getTemplate("chapter.toc.html.chapter");
		chapterTocChapterHeadTP 	= getTemplate("chapter.toc.html.chapter.head");
		chapterTocChapterFootTP 	= getTemplate("chapter.toc.html.chapter.foot");

		bookTocHeadTP 		= getTemplate("book.toc.html.head");
		bookTocFootTP 		= getTemplate("book.toc.html.foot");
		bookTocBookTP 		= getTemplate("book.toc.html.book");

		generate();
	}
		
	protected void beginBible(Map wordMap) throws Exception
	{
		System.out.println("Making HTML Files ...");

		setOutputDir(docName + "/html/");
		
		setOutputFile("toc.hhc", "toc.hhc");
		setOutputFile("book.html", "book.html");

		print(tocHeadTP.getString(wordMap), "toc.hhc");
		print(bookTocHeadTP.getString(wordMap), "book.html");
	}
	
	protected void endBible(Map wordMap) throws Exception
	{
		print(bookTocFootTP.getString(wordMap), "book.html");
		print(tocFootTP.getString(wordMap), "toc.hhc");

		outputFile("index.html", wordMap);
		outputFile("about.html", wordMap);
		outputFile("bible.hhp", wordMap);
		outputFile("index.hhk", wordMap);
		outputFile("converter.html", wordMap);

		System.out.println("HTML Files Generated ...");
		System.out.println();
	}
	
	protected void beginBook(Map wordMap) throws Exception
	{
		String bookNum = (String) wordMap.get("%%booknumtext%%");
		
		setOutputSubDir(bookNum);

		print(tocBookHeadTP.getString(wordMap), "toc.hhc");
		print(bookTocBookTP.getString(wordMap), "book.html");

		setOutputFile("chapter.html", "chapter.html");
		print(chapterTocHeadTP.getString(wordMap), "chapter.html");

		chapterCount = 0;
	}
	
	protected void endBook(Map wordMap) throws Exception
	{
		String bookNum = (String) wordMap.get("%%booknumtext%%");

		print(tocBookFootTP.getString(wordMap), "toc.hhc");

		wordMap.put("%%chaptext%%", "");
		for (int i = chapterCount; i > 0 && i < 5; i++){
			print(chapterTocChapterTP.getString(wordMap), "chapter.html");
		}
		print(chapterTocFootTP.getString(wordMap), "chapter.html");

		outputFile("index.toc.html", bookNum, wordMap);
	}

	protected void beginChapter(Map wordMap) throws Exception
	{
		setOutputFile(wordMap.get("%%chapnumtext%%") + ".html");

		print(headTP.getString(wordMap));
		print(tocChapterTP.getString(wordMap), "toc.hhc");

		if (chapterCount == 0){
			print(chapterTocChapterHeadTP.getString(wordMap), "chapter.html");
		}

		print(chapterTocChapterTP.getString(wordMap), "chapter.html");

		chapterCount++;

		if (chapterCount == 5){
			print(chapterTocChapterFootTP.getString(wordMap), "chapter.html");
			chapterCount = 0;
		}
	}

	protected void endChapter(Map wordMap) throws Exception
	{
		print(footTP.getString(wordMap));
		lastBlank = true;
	}

	protected void verseText(Map wordMap) throws Exception
	{
		print(verseTP.getString(wordMap));
		lastBlank = false;
	}
	
	protected void chapterText(Map wordMap) throws Exception
	{
		if (!lastBlank){
			print(blankTP.getString(wordMap));
		}

		print(chapterTP.getString(wordMap));
		print(blankTP.getString(wordMap));
		
		lastBlank = true;
	}
	
	protected void titleText(Map wordMap) throws Exception
	{
		if (!lastBlank){
			print(blankTP.getString(wordMap));
		}
		print(titleTP.getString(wordMap));

		lastBlank = false;
	}
}
