/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.parser;

public class Verse extends Parser
{
	protected static final int VERSE_LIMIT = 8192;

	protected static final String TAG_NAME = "VERSE";
	protected static final String BOOKTEXT = "BOOKTEXT";
	protected static final String CHAPTEXT = "CHAPTEXT";
	protected static final String DESCTEXT = "DESCTEXT";
	protected static final String VERSTEXT = "VERSTEXT";
	protected int totalCompressedChar;
	
	public Verse(Tokenizer tokenizer, WordRecorder wordRecorder) throws Exception
	{
		checkTag(tokenizer);
		parse(tokenizer, wordRecorder);
	}
	
	public Verse() throws Exception
	{
		addChild("");
	}
	
	public int getTotalChar()
	{
		if (childs.size() > 0){
			return childs.elementAt(0).toString().length();
		} else {
			return 0;
		}
	}
	
	public int getTotalCompressedChar()
	{
		return totalCompressedChar;
	}

	public void setTotalCompressedChar(int totalChar)
	{
		totalCompressedChar = totalChar;
	}

	public String getString()
	{
		if (childs.size() > 0){
			return childs.elementAt(0).toString();
		} else {
			return "";
		}
	}

	public String getTagName()
	{
		return TAG_NAME;
	}

	protected Object parseChild(Tokenizer tokenizer, WordRecorder wordRecorder) throws Exception
	{
		StringBuffer buffer = new StringBuffer();
		Token token = tokenizer.getToken();

		while (!isCloseTag(token)){
		 	if (buffer.length() > VERSE_LIMIT){
		 		throwUnexpectedLongVerse(token);
		 	}
		 	
			if (isOpenTag(token, BOOKTEXT)) {
				buffer.append((char) 0xFFFF);
			} else if (isOpenTag(token, CHAPTEXT)){
				buffer.append((char) 0xFFFE);
			} else if (isOpenTag(token, DESCTEXT)){
				buffer.append((char) 0xFFFD);
			} else if (isOpenTag(token, VERSTEXT)){
				buffer.append((char) 0xFFFC);
			} else if (token.isTag()){
				throwUnexpectedToken(token);
			} else {
				int wordIndex = wordRecorder.getWordIndex(token.toString());
				buffer.append((char) wordIndex);
			}
			token = tokenizer.getToken();
		}
		tokenizer.ungetToken();
		
		return buffer.toString();
	}
	
	protected void throwUnexpectedLongVerse(Token token) throws ParserException
	{
		throw new ParserException(token.getErrorMessage("Unexpected Long of Verse"));
	}
}
