/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.pdbfile;

import java.util.*;
import palmbible.parser.*;
import palmbible.compress.*;
import palmbible.util.*;

public class BookRecords
{
	protected static final int recordSize = 2048;
	protected Vector content = new Vector();
	
	public BookRecords(Book book, WordRecorder wordRecorder, Compressor compressor)
	{
		BookRecord bookRecord = new BookRecord();
		content.addElement(bookRecord);

		int charIndex = 0;
		int totalChapter = book.getTotalChapter();

		ShortShifter shortShifter = new ShortShifter(wordRecorder.getByteShifted());
		for (int i=0; i<totalChapter; i++){
			Chapter chapter = book.getChapter(i);
			int totalVerse = chapter.getTotalVerse();

			for (int j=0; j<totalVerse; j++){
				int compressedCharIndex = 0;
				Verse verse = chapter.getVerse(j);

				int totalChar = verse.getTotalChar();
				String verseString = verse.getString();

				for (int k=0; k<totalChar; k++){
					if (charIndex >= recordSize){
						bookRecord = new BookRecord();
						content.addElement(bookRecord);
						charIndex = 0;
					}

					int newIndex = wordRecorder.getNewWordIndex(verseString.charAt(k));

					int compressedIndex = compressor.getCompressedIndex(newIndex);

					if (compressedIndex != -1){
						compressedCharIndex++;
						int short14 = shortShifter.getShort(compressedIndex);
						if (short14 != -1){
							bookRecord.addShort(short14);
							charIndex ++;
						}
					}
				}
				
				for (int k=0; k < Compressor.COMPRESS_DEPTH; k++){
					int compressedIndex = compressor.getCompressedIndex();
					if (compressedIndex != -1){
						compressedCharIndex++;

						if (charIndex >= recordSize){
							bookRecord = new BookRecord();
							content.addElement(bookRecord);
							charIndex = 0;
						}

						int short14 = shortShifter.getShort(compressedIndex);
						if (short14 != -1){
							bookRecord.addShort(short14);
							charIndex ++;
						}
					}
				}

				verse.setTotalCompressedChar(compressedCharIndex);
			}
		}

		int short14 = shortShifter.flush();
		if (short14 != 0){
			if (charIndex >= recordSize){
				bookRecord = new BookRecord();
				content.addElement(bookRecord);
				charIndex = 0;
			}

			bookRecord.addShort(short14);
			charIndex ++;
		}
	}
	
	public int getTotalRecord()
	{
		return content.size();
	}

	public BookRecord getRecord(int index)
	{
		return (BookRecord) content.elementAt(index);
	}
}
