/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.pdbfile;

import java.io.*;
import java.util.*;
import palmbible.util.*;

public class PDBFile
{
	protected DataOutputStream dataStream;

	protected static final int HEADER_SIZE = 78;

	protected String pdbName;
	protected byte  filler[]           = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	protected short fileAttributes     = 24;
	protected short version            = 1;
	protected int   creationDate;
	protected int   modificationDate;  
	protected int   lastBackupDate;
	protected int   modificationNumber = 0;
	protected int   appInfoArea        = 0;
	protected int   sortInfoArea       = 0;
	protected byte  databaseType[]     = {0,0,0,0};
	protected byte  creatorID[]        = {0,0,0,0};
	protected int   uniqueIDSeed       = 0;
	protected int   nextRecord         = 0;
	protected short numRecs            = 0;

	protected Vector   records;

	public PDBFile(String fileName) throws IOException
	{
		this(fileName, "UNDEFINED");
	}

	public PDBFile(String fileName, String pdbName) throws IOException
	{
		this(fileName, pdbName, "USER");
	}

	public PDBFile(String fileName, String pdbName, String creatorID) throws IOException
	{
		this(fileName, pdbName, creatorID, "DATA");
	}
	
	public PDBFile(String fileName, String pdbName, String creatorID, String databaseType) throws IOException
	{
		File file = new File(fileName);
		dataStream = new DataOutputStream(
					new BufferedOutputStream(
					new FileOutputStream(file)));

		setPDBName(pdbName);
		setCreatorID(creatorID);
		setDatabaseType(databaseType);		

		creationDate = convertPalmDate(new Date());
		modificationDate = creationDate;
		lastBackupDate   = creationDate;

		records = new Vector();
	}

	public String getPDBName() {
		return pdbName;
	}
	
	public void setPDBName(String pdbName){
		this.pdbName = pdbName;
	}

	public short getFileAttributes()
	{
		return fileAttributes;
	}
	
	public void setFileAttributes(short fileAttributes)
	{
		this.fileAttributes = fileAttributes;
	}

	public short getVersion()
	{
		return version;
	}
	
	public void setVersion(short version)
	{
		this.version = version;
	}

	public Date getCreationDate()
	{
		return convertPCDate(creationDate);
	}

	public void setCreationDate(Date creationDate)
	{
		this.creationDate = convertPalmDate(creationDate);
	}

	public Date getModificationDate()
	{
		return convertPCDate(modificationDate);
	}

	public void setModificationDate(Date modificationDate)
	{
		this.modificationDate = convertPalmDate(modificationDate);
	}

	public Date getLastBackupDate()
	{
		return convertPCDate(lastBackupDate);
	}
	
	public void setLastBackupDate(Date lastBackupDate)
	{
		this.lastBackupDate = convertPalmDate(lastBackupDate);
	}

	public String getDatabaseType() {
		return new String(databaseType);
	}

	public void setDatabaseType(String databaseType)
	{
		this.databaseType[0] = (byte) databaseType.charAt(0);
		this.databaseType[1] = (byte) databaseType.charAt(1);
		this.databaseType[2] = (byte) databaseType.charAt(2);
		this.databaseType[3] = (byte) databaseType.charAt(3);
	}

	public String getCreatorID() { return new String(creatorID); }

	public void setCreatorID(String creatorID)
	{
		this.creatorID[0] = (byte) creatorID.charAt(0);
		this.creatorID[1] = (byte) creatorID.charAt(1);
		this.creatorID[2] = (byte) creatorID.charAt(2);
		this.creatorID[3] = (byte) creatorID.charAt(3);
	}

	public short getNumRecs()
	{
		return numRecs;
	}
	
	public void addRecord(PDBRecord pdbRecord)
	{
		records.addElement(pdbRecord);
	}

	public void insertRecord(PDBRecord pdbRecord, int i)
	{
		records.insertElementAt(pdbRecord, i);
	}

	public PDBRecord getRecord(int i)
	{
		return (PDBRecord) records.elementAt(i);
	}

	public void removeRecord(int i)
	{
		records.removeElementAt(i);
	}

	public void close() throws IOException
	{
		writeHeader();
		writeRecordList();
		writeFiller();
		writeRecords();

		dataStream.close();
	}

	protected void writeHeader()
	{
		try {
			dataStream.write(pdbName.getBytes());
			dataStream.write(filler, 0, 32 - pdbName.getBytes().length);
			dataStream.writeShort(fileAttributes);
			dataStream.writeShort(version);
			dataStream.writeInt(creationDate);
			dataStream.writeInt(modificationDate);
			dataStream.writeInt(lastBackupDate);
			dataStream.writeInt(modificationNumber);
			dataStream.writeInt(appInfoArea);
			dataStream.writeInt(sortInfoArea);
			dataStream.write(databaseType);
			dataStream.write(creatorID);
			dataStream.writeInt(uniqueIDSeed);
			dataStream.writeInt(nextRecord);
			dataStream.writeShort(records.size());
		} catch (IOException e) {
			System.err.println("ERROR Writing PDB Header:");
			e.printStackTrace();
		}
	}

	protected void writeRecordList() throws IOException
	{
		int runningTotal = HEADER_SIZE;
		runningTotal += 8 * records.size() + 2;

		for (int i = 0; i < records.size(); i++) {
			PDBRecord pdbRecord = (PDBRecord) records.elementAt(i);

			dataStream.writeInt(runningTotal);
			dataStream.writeByte(pdbRecord.getAttributes());
			dataStream.write(filler, 0, 3);

			runningTotal += pdbRecord.size();
		}
	}

	protected void writeFiller() throws IOException
	{
	    dataStream.write(filler, 0, 2);
	}

	protected void writeRecords() throws IOException
	{
		for (int i = 0; i < records.size(); i++) {
			PDBRecord pdbRecord = (PDBRecord) records.elementAt(i);
			ByteBuffer byteBuffer = pdbRecord.getBytes();
			byteBuffer.reset();
			
			for (int j=0; j < pdbRecord.size(); j++) {
        		dataStream.write(byteBuffer.read());
        	}
		}
	}

	private int convertPalmDate(Date date)
	{
		Calendar calenderNow = Calendar.getInstance();
		calenderNow.set(1904, 0, 1);
		Date dateSince = new Date(date.getTime() + calenderNow.getTime().getTime());
		return (int) dateSince.getTime() / 1000;
	}

	private Date convertPCDate(int date)
	{
		Calendar calenderNow = Calendar.getInstance();
		calenderNow.set(1904, 0, 1);
		return new Date(date * 1000 - calenderNow.getTime().getTime());
	}
}

