/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.pdbfile;

import palmbible.parser.*;
import palmbible.compress.*;
import palmbible.util.*;

public class WordIndexRecord extends PDBRecord
{
	protected static final int WORD_INDEX_LIMIT = 4095;

	public WordIndexRecord(WordRecorder wordRecorder, Compressor compressor)
	{
		int i, totalWordInEntry, totalIndex;
	
		ByteBuffer byteBuffer = new ByteBuffer();
		totalIndex = 0;

		int totalWord = wordRecorder.getWordCount();
		int length = 0;
		totalWordInEntry = 0;
		for (i=0; i<totalWord; i++){
			String word = wordRecorder.getNewWordString(i + 1);
			
			if (length == 0){
				length = word.length();
			}

			if (word.length() != length || totalWordInEntry >= WORD_INDEX_LIMIT){
				byteBuffer.addShort(length);
				byteBuffer.addShort(totalWordInEntry);
				byteBuffer.addShort(0);
				length = word.length();
				totalIndex++;
				totalWordInEntry = 0;
			}

			totalWordInEntry++;
		}
		
		byteBuffer.addShort(length);
		byteBuffer.addShort(totalWordInEntry);
		byteBuffer.addShort(0);
		totalIndex++;
		
		int totalCompressedWord = compressor.getTotalCompressedWord();
		length = 0;

		totalWordInEntry = 0;
		for (i=0; i<totalCompressedWord; i++){
			String word = compressor.getCompressedWord(i + 1);
			
			if (length == 0){
				length = word.length();
			}

			if (word.length() != length || totalWordInEntry >= WORD_INDEX_LIMIT) {
				byteBuffer.addShort(length * 2);
				byteBuffer.addShort(totalWordInEntry);
				byteBuffer.addByte(1);
				byteBuffer.addByte(1);
				length = word.length();
				totalIndex++;
				totalWordInEntry = 0;
			}

			totalWordInEntry++;
		}
		
		byteBuffer.addShort(length * 2);
		byteBuffer.addShort(totalWordInEntry);
		byteBuffer.addByte(1);
		byteBuffer.addByte(1);
		totalIndex++;
		content.addShort(totalIndex);
		content.addByteBuffer(byteBuffer);
	}
}
