/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.util;

import java.util.*;

public class ByteBuffer
{
	protected static String enc;

	protected Vector elements = new Vector();
	protected int elementIndex;

	protected byte[] currentBytes;
	protected int byteIndex;
	protected int size = 0;

	public ByteBuffer()
	{
	}
	
	public static void setEncoding(String enc)
	{
		ByteBuffer.enc = enc;
	}
	
	public void addByte(int value)
	{
		byte[] element = new byte[1];
		element[0] = (byte)( value & 0x000000FF);
		elements.addElement(element);
		size += 1;
	}

	public void addByte(String string)
	{
		addByte(string, string.length());
	}
	
	public void addByte(String string, int length)
	{
		int strLen = string.length();
	
		byte[] element = new byte[length];
		for (int i=0; i<length; i++){
			if (i < strLen){
				element[i] = (byte) string.charAt(i);
			} else {
				element[i] = 0;
			}
		}
		
		elements.addElement(element);
		size += length;
	}

	public void addShort(int value)
	{
		byte[] shortValue = new byte[2];
		
		shortValue[0] = (byte) ((value & 0x0000FF00) >> 8);
		shortValue[1] = (byte) (value & 0x000000FF);
		
		elements.addElement(shortValue);
		size += 2;
	}
	
	public void addInt(int value)
	{
		byte[] intValue = new byte[4];
		
		intValue[0] = (byte) ((value & 0xFF000000) >> 24);
		intValue[1] = (byte) ((value & 0x00FF0000) >> 16);
		intValue[2] = (byte) ((value & 0x0000FF00) >> 8);
		intValue[3] = (byte) (value & 0x000000FF);
		
		elements.addElement(intValue);
		size += 4;
	}
	
	public void addString(String string, int length) throws Exception
	{
		byte[] element = new byte[length];

		byte[] encByte = string.getBytes(enc);
		int strLen = encByte.length;
		
		for (int i=0; i<length - 1; i++){
			if (i < strLen){
				element[i] = encByte[i];
			} else {
				element[i] = 0;
			}
		}
		element[length - 1] = 0;

		elements.addElement(element);
		size += length;
	}
	
	public void addByteBuffer(ByteBuffer byteBuffer)
	{
		Vector vector = byteBuffer.elements;
		int totalElements = vector.size();
		
		for (int i=0; i<totalElements; i++){
			elements.addElement(vector.elementAt(i));
		}
		
		size += byteBuffer.size;
	}
	
	public void reset()
	{
		currentBytes = (byte[]) elements.firstElement();
		byteIndex = 0;
		elementIndex = 0;
	}

	public byte read()
	{
		byte result = 0;
		
		if (byteIndex < currentBytes.length) {
			result = currentBytes[byteIndex];
			byteIndex++;
		} else {
			if (elementIndex < elements.size() - 1) {
				elementIndex++;
				currentBytes = (byte[]) elements.elementAt(elementIndex);
				result = currentBytes[0];
				byteIndex = 1;
			}
		}
		
		return result;
	}

	public int size()
	{
		return size;
	}
}
