/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#define SECT1 __attribute__((section("SECT1")))

// Constants

#define winNull winLeft;				// no move value
#define	viewUpdateCode 100				// update code for update event

#define moveTH 2						// move threadshold
#define	isUpperView true				// indicate upperView
#define	isLowerView false				// indicate lowerView
#define upperBuffer	16					// lines to buffer in viewer
#define middleBuffer 14					// lines to buffer in viewer
#define lowerBuffer	12					// lines to buffer in viewer
#define assumeLen	1					// assume lines of verse

#define minLineSpace 11
#define midLineSpace 13
#define highLineSpace 15

#define defaultLineSpace midLineSpace
#define defaultWordSpace 0

#define searchVerseWidth 50
#define totalResultLine 10
#define searchBufferSize 192
#define searchRecordBreak 20

#define screenHeight 145
#define maxTitleWidth 80

#define titleStateSeconds 60
#define searchReturnTick 1

// Other Components Header

#include "colorbox.h"
#include "info.h"

#include "recordReader.h"

#include "decode.h"
#include "render.h"

#include "reader.h"
#include "viewer.h"

#include "history.h"
#include "bookmark.h"

#include "config.h"
#include "database.h"


// Global Variables

Boolean os35;
Boolean restart;

FormPtr modalForm;

RectangleType fullBound;
RectangleType upperBound;
RectangleType lowerBound;

ViewerType upperViewer;
ViewerType lowerViewer;

UInt16 lineSpace;
UInt16 wordSpace;
Boolean dualView;

Boolean noBlankNormal;
Boolean noBlankDual;

Boolean boldBook;
Boolean boldDesc;
Boolean boldChapter;
Boolean boldVerse;
Boolean boldText;

Boolean useVBold;
Boolean vBold;

HistoryEntryType historyEntry[backHistory];
HistoryType	history;

IndexedColorType bookColorIndex;
IndexedColorType descColorIndex;
IndexedColorType chapterColorIndex;
IndexedColorType verseColorIndex;
IndexedColorType textColorIndex;

IndexedColorType returnColorIndex;

Boolean startSearch;
Boolean searching;

Boolean inSearch;
UInt16 searchVersion;
BookmarkType searchFrom;
BookmarkType searchTo;
BookmarkType searchProgress;
UInt16 resultLine;
BookmarkType result[totalResultLine];

MemHandle searchTextHandle;

Boolean doubleBuffered;

// Global Functions (located in main.c)

void gotoSearch() SECT1;

UInt16 getObjectIndex(UInt16 objID) SECT1;
void* getObjectPtr(UInt16 objID) SECT1;

void setControlValue(UInt16 objID, Boolean value) SECT1;
Boolean getControlValue(UInt16 objID) SECT1;

void setPopUp(UInt16 popID, UInt16 listID) SECT1;

void setScroll(UInt16 sclID, UInt16* value, UInt16 total) SECT1;

void setFieldSelection(UInt16 fldID, UInt16 start, UInt16 length) SECT1;
void setFieldScrollValue(UInt16 fldID, UInt16 sclID) SECT1;

void releaseFieldFocus(UInt16 objID) SECT1;
void scrollField(UInt16 fldID, UInt16 lines, WinDirectionType direction) SECT1;

void setControlUsable(UInt16 objID, Boolean value) SECT1;

void setListChoices(UInt16 objID, ListContainPtr list) SECT1;

void setListSelection(UInt16 objID, ListContainPtr list, Int16 selectValue) SECT1;
Int16 getListSelection(UInt16 objID, ListContainPtr list) SECT1;
char* getListSelectionText(UInt16 listID, ListContainPtr list) SECT1;

void setField(UInt16 objID, MemHandle fieldText) SECT1;
void setFieldValue(UInt16 objID, MemHandle field, UInt16 value) SECT1;
UInt16 getFieldValue(UInt16 objID, MemHandle field) SECT1;

Int16 getListItem(ListContainPtr list, Int16 index) SECT1;
Int16 getListIndex(ListContainPtr list, Int16 item) SECT1;

void ListPtrFree(ListContainPtr list) SECT1;
ListContainPtr ListPtrNew(UInt16 totalValue) SECT1;

void VersePtrFree(VersePtr verse) SECT1;
VersePtr VersePtrNew() SECT1;

void sortListPtr(ListContainPtr list, Boolean name, Boolean acc) SECT1;

// Event Functions

Boolean ApplicationHandleMenu(UInt16 menuID);
Boolean frmMain_HandleEvent(EventPtr event);
Boolean frmGoto_HandleEvent(EventPtr event) SECT1;
Boolean frmPref_HandleEvent(EventPtr event) SECT1;
Boolean frmSearch_HandleEvent(EventPtr event) SECT1;
Boolean frmSearchResult_HandleEvent(EventPtr event) SECT1;
Boolean frmBookmark_HandleEvent(EventPtr event) SECT1;
Boolean frmAbout_HandleEvent(EventPtr event) SECT1;
Boolean frmSelectColor_HandleEvent(EventPtr event) SECT1;
Boolean frmDeleteVer_HandleEvent(EventPtr event) SECT1;
Boolean frmVersionInfo_HandleEvent(EventPtr event) SECT1;