/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"

static SECT1 DmOpenRef configDB;

static SECT1 void copyMem(void* source, void* dest, UInt16 bytes)
{
	UInt16 i;
	UInt8* sourceByte;
	UInt8* destByte;
	
	sourceByte = source;
	destByte = dest;
	
	for (i=0; i<bytes; i++){
		destByte[i] = sourceByte[i];
	}
}

static SECT1 void setDefaultVersion()
{
	ListContainPtr books;
	char* searchTextPtr;

	upperViewer.version = 1;
	books = InfGetSimpleBookList(upperViewer.version);
	
	upperViewer.book = books->numList[0];
	upperViewer.chapter = 1;
	upperViewer.verse = 1;
	upperViewer.offset = 0;
	
	lowerViewer.version = 1;
	lowerViewer.book = upperViewer.book;
	lowerViewer.chapter = 1;
	lowerViewer.verse = 1;
	lowerViewer.offset = 0;

	ListPtrFree(books);

	searchVersion = upperViewer.version;

	searchFrom.book = upperViewer.book;
	searchFrom.chapter = upperViewer.chapter;
	searchFrom.verse = upperViewer.verse;

	searchTo.book = upperViewer.book;
	searchTo.chapter = upperViewer.chapter;
	searchTo.verse = upperViewer.verse;
	
	searchTextPtr = MemHandleLock(searchTextHandle);
	searchTextPtr[0]=0;
	searchTextPtr[0]=0;
	MemHandleUnlock(searchTextHandle);
}

static SECT1 void setDefaultConfig()
{
	RGBColorType color;
	
	lineSpace = defaultLineSpace;
	wordSpace = defaultWordSpace;
	dualView = false;

	noBlankNormal = false;
	noBlankDual = true;
	boldBook = false;
	boldDesc = false;
	boldChapter = true;
	boldVerse = true;
	boldText = false;
	useVBold = true;
	
	if (os35){
    	color.r = 0;
    	color.g = 0;
    	color.b = 96;
    	bookColorIndex = WinRGBToIndex(&color);
    
    	color.r = 96;
    	color.g = 96;
    	color.b = 192;
    	chapterColorIndex = WinRGBToIndex(&color);
    
    	color.r = 96;
    	color.g = 96;
    	color.b = 96;
    	descColorIndex = WinRGBToIndex(&color);
    
    	color.r = 192;
    	color.g = 0;
    	color.b = 0;
    	verseColorIndex = WinRGBToIndex(&color);
    
    	textColorIndex = UIColorGetTableEntryIndex(UIFieldText);
	} else {
    	bookColorIndex = 0;
    	chapterColorIndex = 0;
    	descColorIndex = 0;
    	verseColorIndex = 0;
    	textColorIndex = 0;
	}
	
	setDefaultVersion();
}

static SECT1 void saveConfig()
{
	char* searchTextPtr;

	ConfigType configRecord;

	configRecord.lineSpace = lineSpace;
	configRecord.wordSpace = wordSpace;

	configRecord.noBlankNormal = noBlankNormal;
	configRecord.noBlankDual = noBlankDual;

	if (os35){
    	WinIndexToRGB(bookColorIndex, &configRecord.bookColor);
    	WinIndexToRGB(descColorIndex, &configRecord.descColor);
    	WinIndexToRGB(chapterColorIndex, &configRecord.chapterColor);
    	WinIndexToRGB(verseColorIndex, &configRecord.verseColor);
    	WinIndexToRGB(textColorIndex, &configRecord.textColor);
	} else {
    	configRecord.bookColor.r = 0;
    	configRecord.chapterColor.r = 0;
    	configRecord.descColor.r = 0;
    	configRecord.verseColor.r = 0;
    	configRecord.textColor.r = 0;

    	configRecord.bookColor.g = 0;
    	configRecord.chapterColor.g = 0;
    	configRecord.descColor.g = 0;
    	configRecord.verseColor.g = 0;
    	configRecord.textColor.g = 0;

    	configRecord.bookColor.b = 0;
    	configRecord.chapterColor.b = 0;
    	configRecord.descColor.b = 0;
    	configRecord.verseColor.b = 0;
    	configRecord.textColor.b = 0;
	}

	configRecord.boldBook = boldBook;
	configRecord.boldDesc = boldDesc;
	configRecord.boldChapter = boldChapter;
	configRecord.boldVerse = boldVerse;
	configRecord.boldText = boldText;
	configRecord.useVBold = useVBold;

	configRecord.dualView = dualView;
	configRecord.upperViewer = upperViewer;
	configRecord.lowerViewer = lowerViewer;

	configRecord.inSearch = inSearch;
	configRecord.searchVersion = searchVersion;
	configRecord.searchFrom = searchFrom;
	configRecord.searchTo = searchTo;
	configRecord.searchProgress = searchProgress;
	configRecord.resultLine = resultLine;
	copyMem(result, configRecord.result, sizeof(BookmarkType) * totalResultLine);
	
	configRecord.lastTotalVersion = DmxGetTotalVersions();

	DmxAddRecord(configDB, dmMaxRecordIndex, &configRecord, sizeof(ConfigType));
	
	searchTextPtr = MemHandleLock(searchTextHandle);
	DmxAddRecord(configDB, dmMaxRecordIndex, searchTextPtr, StrLen(searchTextPtr) + 1);
	MemHandleUnlock(searchTextHandle);
}

static SECT1 void loadConfig()
{
	ConfigPtr configPtr;
	char* storeTextPtr;
	char* searchTextPtr;
	RGBColorType color;

	configPtr = DmxLockRecord(configDB, 0);

	lineSpace = configPtr->lineSpace;
	wordSpace = configPtr->wordSpace;

	noBlankNormal = configPtr->noBlankNormal;
	noBlankDual = configPtr->noBlankDual;
	boldBook = configPtr->boldBook;
	boldDesc = configPtr->boldDesc;
	boldChapter = configPtr->boldChapter;
	boldVerse = configPtr->boldVerse;
	boldText = configPtr->boldText;
	useVBold = configPtr->useVBold;

	if (os35){
    	color = configPtr->bookColor;
    	bookColorIndex = WinRGBToIndex(&color);
    	color = configPtr->descColor;
    	descColorIndex = WinRGBToIndex(&color);
    	color = configPtr->chapterColor;
    	chapterColorIndex = WinRGBToIndex(&color);
    	color = configPtr->verseColor;
    	verseColorIndex = WinRGBToIndex(&color);
    	color = configPtr->textColor;
    	textColorIndex = WinRGBToIndex(&color);
	} else {
    	bookColorIndex = 0;
    	chapterColorIndex = 0;
    	descColorIndex = 0;
    	verseColorIndex = 0;
    	textColorIndex = 0;
	}

	dualView = configPtr->dualView;
	upperViewer = configPtr->upperViewer;
	lowerViewer = configPtr->lowerViewer;

	upperViewer.totalLines = 0;
	upperViewer.begin = NULL;
	
	lowerViewer.totalLines = 0;
	lowerViewer.begin = NULL;
	
	if (configPtr->lastTotalVersion != DmxGetTotalVersions()){
		setDefaultVersion();
	}

	inSearch = configPtr->inSearch;
	searchVersion = configPtr->searchVersion;
	searchFrom = configPtr->searchFrom;
	searchTo = configPtr->searchTo;
	searchProgress = configPtr->searchProgress;
	resultLine = configPtr->resultLine;
	
	copyMem(configPtr->result, result, sizeof(BookmarkType) * totalResultLine);

	DmxUnlockRecord();
	
	storeTextPtr = DmxLockRecord(configDB, 1);
	searchTextPtr = MemHandleLock(searchTextHandle);

	copyMem(storeTextPtr, searchTextPtr, StrLen(storeTextPtr) + 1);
	MemHandleUnlock(searchTextHandle);
	DmxUnlockRecord();
}

void CfgOpenConfig()
{
    Boolean supportsColor;
	UInt32 romVersion;
	UInt16 totalConfig;

	FtrGet(sysFtrCreator,
	sysFtrNumROMVersion, &romVersion);

	if (romVersion >= 0x03503000){
	    WinScreenMode(winScreenModeGetSupportsColor, NULL, NULL,
			NULL, &supportsColor);
		
		if (supportsColor){
			os35 = true;
		} else {
			os35 = false;
		}
	} else {
		os35 = false;
	}

	searchTextHandle = MemHandleNew(64);

	configDB = DmxOpenConfig("Config");
	totalConfig = DmNumRecords(configDB);
	
	if (totalConfig > 0){
		loadConfig();
	} else {
		setDefaultConfig();
		saveConfig();
	}
	
	DmCloseDatabase(configDB);
	
	doubleBuffered = false;
}

void CfgCloseConfig()
{
	UInt16 i;

	configDB = DmxOpenConfig("Config");
	DmRemoveRecord(configDB, 1);
	DmRemoveRecord(configDB, 0);
	saveConfig();
	DmCloseDatabase(configDB);

	MemHandleFree(searchTextHandle);
}

void CfgSetDefaultVersion()
{
	setDefaultVersion();
}

