/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"

static SECT1 UInt16 getIncHistory(UInt16 history)
{
	history++;
	
	if (history >= backHistory){
		history = backHistory - 1;
	}
	
	return history;
}

static SECT1 UInt16 getDecHistory(UInt16 history)
{
	history--;
	
	if (history < 0){
		history = 0;
	}
	
	return history;
}

static SECT1 UInt16 getNextHistory(UInt16 history)
{
	history++;

	if (history >= backHistory){
		history = 0;
	}
	
	return history; 
}

static SECT1 UInt16 getPrevHistory(UInt16 history)
{
	history--;

	if (history < 0){
		history = backHistory - 1;
	}

	return history; 
}

static SECT1 void saveCurrent()
{
	HistoryEntryType localHistoryEntry;

	localHistoryEntry.version = upperViewer.version;
	localHistoryEntry.book = upperViewer.book;
	localHistoryEntry.chapter = upperViewer.chapter;
	localHistoryEntry.verse = upperViewer.verse;
	localHistoryEntry.offset = upperViewer.offset;
	localHistoryEntry.version2 = lowerViewer.version;
	localHistoryEntry.dualView = dualView;

	historyEntry[history.currentHistory] = localHistoryEntry;
}

static SECT1 void restoreCurrent()
{
	HistoryEntryType localHistoryEntry;

	localHistoryEntry = historyEntry[history.currentHistory];

	upperViewer.version = localHistoryEntry.version;
	upperViewer.book = localHistoryEntry.book;
	upperViewer.chapter = localHistoryEntry.chapter;
	upperViewer.verse = localHistoryEntry.verse;
	upperViewer.offset = localHistoryEntry.offset;
	lowerViewer.version = localHistoryEntry.version2;
	dualView = localHistoryEntry.dualView;
}

// Public Functions

void HtyOpenHistory()
{
	history.currentHistory = 0;
	history.totalForward = 0;
	history.totalBack = 0;
}

void HtyCloseHistory()
{
}

void HtyPushHistory()
{
	saveCurrent();

	history.totalForward = 0;
	history.totalBack = getIncHistory(history.totalBack);
	history.currentHistory = getNextHistory(history.currentHistory);
}

void HtyForwardHistory()
{
	if (history.totalForward > 0){
		saveCurrent();
		history.totalForward = getDecHistory(history.totalForward);
		history.totalBack = getIncHistory(history.totalBack);
		history.currentHistory = getNextHistory(history.currentHistory);
		restoreCurrent();
	}
}

void HtyBackHistory()
{
	if (history.totalBack > 0){
		saveCurrent();
		history.totalForward = getIncHistory(history.totalForward);
		history.totalBack = getDecHistory(history.totalBack);
		history.currentHistory = getPrevHistory(history.currentHistory);
		restoreCurrent();
	}
}
