/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

// Constants

#define infVerInfoRecord 0

#define infCopyProtected	0x01
#define infByteNotShifted	0x02
#define infRightAligned		0x04

// Structure Definition

typedef struct BookInfoType { 
	UInt16 bookNum;
	UInt16 bookIndex;
	UInt16 totalBookRec;
	char simpleName[8];
	char complexName[32];
} BookInfoType;
typedef BookInfoType *BookInfoPtr;

typedef struct VersionInfoType {
	char versionName[16];
	char versionInfo[128];
	char sepChar;
	UInt8 versionAttrib;
	UInt16 wordIndex;
	UInt16 totalWordRec;
	UInt16 totalBooks;
	BookInfoType bookInfo[1]; // bookInfo array
} VersionInfoType;
typedef VersionInfoType *VersionInfoPtr;

typedef struct ListContainType {
	char **nameList;
	UInt16 *numList;
	UInt16 totalValue;
} ListContainType;
typedef ListContainType *ListContainPtr;

struct VerseType;
typedef struct VerseType *VersePtr;
typedef struct VerseType {
	UInt16 lines;
	char **textLines;

	UInt16 chapter;
	UInt16 verse;

	VersePtr prev;
	VersePtr next;
} VerseType;

// Public Functions

void InfOpenInfo();
void InfCloseInfo();

char InfGetSepChar(UInt16 verion);

Boolean InfGetCopyProtected(UInt16 version);
Boolean InfGetByteShifted(UInt16 version);
Boolean InfGetRightAligned(UInt16 version);

char* InfGetVersionName(UInt16 version);
char* InfGetVersionInfo(UInt16 version);

ListContainPtr InfGetVersionList();

char* InfGetSimpleBookName(UInt16 version, UInt16 book);
ListContainPtr InfGetSimpleBookList(UInt16 version);

char* InfGetComplexBookName(UInt16 version, UInt16 book);
ListContainPtr InfGetComplexBookList(UInt16 version);

UInt16 InfGetWordRecordIndex(UInt16 version);
UInt16 InfGetWordTotalRecord(UInt16 version);

UInt16 InfGetBookRecordIndex(UInt16 version, UInt16 book);
UInt16 InfGetBookTotalRecord(UInt16 version, UInt16 book);

UInt16 InfGetTotalVersion();
UInt16 InfGetTotalBook(UInt16 version);
UInt16 InfGetTotalChapter(UInt16 version, UInt16 book);
UInt16 InfGetTotalVerse(UInt16 version, UInt16 book, UInt16 chapter);

char* InfGetSimpleFormattedVerse(UInt16 version, UInt16 book, 
	UInt16 chapter, UInt16 verse);
char* InfGetComplexFormattedVerse(UInt16 version, UInt16 book, 
	UInt16 chapter, UInt16 verse);

