/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"
#include "PalmBible_res.h"

static UInt16 normalReturnTick;

static int StartApplication(void)
{
	DmxOpenDatabase();
	if (DmxGetTotalVersions() > 0){
		CfgOpenConfig();
		RenOpenRender();
		InfOpenInfo();
		BmkOpenBookmark();
		HtyOpenHistory();

		FrmGotoForm(frmMain);
		return 0;
	} else {
		FrmAlert(altNoVersion);
		return 1;
	}
}

static Boolean ApplicationHandleEvent(EventPtr event)
{
	UInt16 formID;
	FormPtr form;
	Boolean handled = false;
	
	switch (event->eType)
	{
		case menuEvent:
			handled = ApplicationHandleMenu(event->data.menu.itemID);
			break;
		case frmLoadEvent:
			formID = event->data.frmLoad.formID;
			form = FrmInitForm(formID);
			FrmSetActiveForm(form);		

			switch (formID)
			{
				case frmVersionInfo:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmVersionInfo_HandleEvent);
					break;
				case frmDeleteVer:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmDeleteVer_HandleEvent);
					break;
				case frmSelectColor:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmSelectColor_HandleEvent);
					break;
				case frmPref:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmPref_HandleEvent);
					break;
				case frmBookmark:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmBookmark_HandleEvent);
					break;
				case frmSearchResult:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmSearchResult_HandleEvent);
					break;
				case frmAbout:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmAbout_HandleEvent);
					break;
				case frmSearch:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmSearch_HandleEvent);
					break;
				case frmGoto:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmGoto_HandleEvent);
					break;
				case frmMain:
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmMain_HandleEvent);
					break;
				default:
					break;
			}
			handled = true;
			break;
		default:
			break;
	}

	return handled;
}

static void EventLoop(void)
{
	UInt16 error;
	EventType event;

	searching = false;
	restart = false;
	
	normalReturnTick = SysTicksPerSecond() * titleStateSeconds;

	do
	{
		if (searching){
			EvtGetEvent(&event, searchReturnTick);
		} else {
			EvtGetEvent(&event, normalReturnTick);
		}
	
		if (!SysHandleEvent(&event))
		{
			if (!MenuHandleEvent(0, &event, &error))
			{
				if (!ApplicationHandleEvent(&event))
					FrmDispatchEvent(&event);
			}
		}
	}
	while (!restart && event.eType != appStopEvent);
}

static void StopApplication(void)
{
	FrmCloseAllForms();

	HtyCloseHistory();
	BmkCloseBookmark();
	InfCloseInfo();
	RenCloseRender();
	CfgCloseConfig();
	DmxCloseDatabase();
}

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	UInt32 error;
	
	if (cmd == sysAppLaunchCmdNormalLaunch)
 	{
		error = StartApplication();
		if (error){
			return error;
		}
		
		EventLoop();
		StopApplication();
	}

	return 0;
}

/****************************************************************************
	Private Functions
****************************************************************************/

static SECT1 UInt16 forwardLookup(UInt16* numLookupList, UInt16 totalValue, UInt16 value)
{
	UInt16 i;
	
	if (numLookupList == NULL){
		return value;
	} else {
		for (i=0; i<totalValue; i++){
			if (value == numLookupList[i]){
				return i;
			}
		}
		return 0;
	}
}

static SECT1 UInt16 reverseLookup(UInt16* numLookupList, UInt16 totalValue, UInt16 value)
{
	if (numLookupList == NULL){
		return value;
	} else {
		if (value < totalValue){
			return numLookupList[value];
		} else {
			return 1;
		}
	}
}

static SECT1 void swapListElement(ListContainPtr list, UInt16 i)
{
	char* tempName;
	UInt16 tempNum;

	tempName = list->nameList[i];
	list->nameList[i] = list->nameList[i + 1];
	list->nameList[i + 1] = tempName;

	tempNum = list->numList[i];
	list->numList[i] = list->numList[i + 1];
	list->numList[i + 1] = tempNum;
}

/****************************************************************************
	Public Functions
****************************************************************************/

void gotoSearch()
{
	if (inSearch && searchVersion == upperViewer.version){
		startSearch = false;
		FrmGotoForm(frmSearchResult);
	} else {
		FrmGotoForm(frmSearch);
	}
}

UInt16 getObjectIndex(UInt16 objID)
{
	return FrmGetObjectIndex(modalForm, objID);
}

void* getObjectPtr(UInt16 objID)
{
	UInt16 objIndex;
	FormPtr objPtr;
	
	objIndex = getObjectIndex(objID);
	objPtr = FrmGetObjectPtr(modalForm, objIndex);
	return objPtr;
}

void setControlValue(UInt16 objID, Boolean value)
{
	UInt16 objIndex;
	
	objIndex = getObjectIndex(objID);
	FrmSetControlValue(modalForm, objIndex, value ? 1:0);
}

Boolean getControlValue(UInt16 objID)
{
	UInt16 objIndex;
	
	objIndex = getObjectIndex(objID);
	return (FrmGetControlValue(modalForm, objIndex) > 0);
}

void setPopUp(UInt16 popID, UInt16 listID)
{
	ListPtr list;
	ControlPtr popUp;

	char* oldText;
	char* selectionText;
	char* selectionTextTrimmed;
	UInt16 selection;

	list = getObjectPtr(listID);
	selection = LstGetSelection(list);
	selectionText = LstGetSelectionText(list, selection);
	
	popUp = getObjectPtr(popID);
	CtlSetLabel(popUp, selectionText);
}

void setScroll(UInt16 sclID, UInt16* value, UInt16 total)
{
	ScrollBarPtr scroll;
	UInt16 min, max;
	
	scroll = getObjectPtr(sclID);
	
	SclSetScrollBar(scroll, *value, 1, total, 1);
	SclGetScrollBar(scroll, value, &min, &total, &max);
}

void setFieldSelection(UInt16 fldID, UInt16 start, UInt16 length)
{
	FieldPtr field;
	
	field = getObjectPtr(fldID);
	FldSetSelection(field, start, length);
}

void setFieldScrollValue(UInt16 fldID, UInt16 sclID)
{
	FieldPtr field;
	ScrollBarPtr scroll;
	UInt16 maxValue, scrollPos, textHeight, fieldHeight;

	field = getObjectPtr(fldID);
	scroll = getObjectPtr(sclID);

    FldGetScrollValues(field, &scrollPos, &textHeight, &fieldHeight);

    if (textHeight > fieldHeight)
        maxValue = textHeight - fieldHeight;
    else if (scrollPos)
        maxValue = scrollPos;
    else
        maxValue = 0;
    
    SclSetScrollBar (scroll, scrollPos, 0, maxValue, fieldHeight - 1);
}

void scrollField(UInt16 objID, UInt16 lines, WinDirectionType direction)
{
	FieldPtr field;

	field = getObjectPtr(objID);
	FldScrollField (field, lines, direction);
}

void releaseFieldFocus(UInt16 objID)
{
	FieldPtr field;
	
	field = getObjectPtr(objID);
	FldReleaseFocus(field);
}

void setControlUsable(UInt16 objID, Boolean usable)
{
	FormPtr controlPtr;
	UInt16 objIndex;
	
	objIndex = getObjectIndex(objID);

	if (usable){
		FrmShowObject(modalForm, objIndex);
	} else {
		FrmHideObject(modalForm, objIndex);
	}
}

void setListChoices(UInt16 objID, ListContainPtr list)
{
	ListPtr listPtr;
	
	listPtr = getObjectPtr(objID);
	LstSetListChoices(listPtr, list->nameList, list->totalValue);
	LstDrawList(listPtr);
}

Int16 getListItem(ListContainPtr list, Int16 selection)
{
	if (list != NULL){
		return reverseLookup(list->numList, list->totalValue, selection);
	} else {
		return selection;
	}
}

Int16 getListIndex(ListContainPtr list, Int16 selectValue)
{
	if (list != NULL){
		return forwardLookup(list->numList, list->totalValue, selectValue);
	} else {
		return selectValue;
	}
}

void setListSelection(UInt16 objID, ListContainPtr list, Int16 selectValue)
{
	ListPtr listPtr;
	UInt16 listSelectValue;
	
	listPtr = getObjectPtr(objID);
	if (list != NULL){
		listSelectValue = forwardLookup(list->numList, list->totalValue, selectValue);
	} else {
		listSelectValue = selectValue;
	}

	LstSetSelection(listPtr, listSelectValue);
	if (listSelectValue != -1){
		LstMakeItemVisible(listPtr, listSelectValue);
	}
}

char* getListSelectionText(UInt16 listID, ListContainPtr list)
{
	ListPtr listPtr;
	UInt16 selection;
	char* selectionText;
	
	listPtr = getObjectPtr(listID);
	selection = LstGetSelection(listPtr);
	
	if (list != NULL){
		selectionText = list->nameList[selection];
	} else {
		selectionText = LstGetSelectionText(listPtr, selection);
	}
	
	return selectionText;
}

Int16 getListSelection(UInt16 objID, ListContainPtr list)
{
	ListPtr listPtr;
	UInt16 selection;
	
	listPtr = getObjectPtr(objID);
	selection = LstGetSelection(listPtr);
	
	if (list != NULL){
		return reverseLookup(list->numList, list->totalValue, selection);
	} else {
		return selection;
	}
}

void setField(UInt16 objID, MemHandle fieldText)
{
	FieldPtr fieldPtr;

	fieldPtr = getObjectPtr(objID);

	FldSetTextHandle(fieldPtr, fieldText);
	FldDrawField(fieldPtr);
}

void setFieldValue(UInt16 objID, MemHandle fieldText, UInt16 value)
{
	char* textValue;

	if (fieldText != NULL){
		textValue = MemHandleLock(fieldText);
		StrPrintF(textValue, "%d", value);
		MemHandleUnlock(fieldText);
	}

	setField(objID, fieldText);
}

UInt16 getFieldValue(UInt16 objID, MemHandle fieldText)
{
	FieldPtr fieldPtr;
	char* textValue;
	UInt16 numValue;

	textValue = MemHandleLock(fieldText);
	numValue = StrAToI(textValue);

	MemHandleUnlock(fieldText);
		
	return numValue;
}

void sortListPtr(ListContainPtr list, Boolean name, Boolean acc)
{
	UInt16 i, j;
	UInt16 totalValue;
	Int16 accValue;
	
	totalValue = list->totalValue;
	accValue = acc?1:-1;
	
	if (totalValue > 1){
		for (i=0; i < totalValue; i++){
			for (j=0; j < (totalValue - 1); j++){
				if (name){
					if (StrCompare(list->nameList[j], list->nameList[j + 1]) * accValue > 0) {
						swapListElement(list, j);
					}
				} else {
					if (list->numList[j] > list->numList[j+1] ? -1:1 * accValue > 0){
						swapListElement(list, j);
					}
				}
			}
		}
	}
}

void ListPtrFree(ListContainPtr list)
{
	UInt16 i;
	
	if (list->totalValue > 0){
		for (i=0; i < list->totalValue;i++)
		{
			MemPtrFree(list->nameList[i]);
		}

		MemPtrFree(list->numList);
		MemPtrFree(list->nameList);
	}
	MemPtrFree(list);
}

ListContainPtr ListPtrNew(UInt16 totalValue)
{
	ListContainPtr list;
	
	list = MemPtrNew(sizeof(ListContainType));
	if (totalValue > 0){
		list->nameList = MemPtrNew(sizeof(char*) * totalValue);
		list->numList = MemPtrNew(sizeof(UInt16) * totalValue);
	}
	list->totalValue = totalValue;

	return list;
}

void VersePtrFree(VersePtr verse)
{
	UInt16 i;

	for (i=0; i<verse->lines; i++){
		MemPtrFree(verse->textLines[i]);
	}

	if (verse->lines > 0){
		MemPtrFree(verse->textLines);
	}

	MemPtrFree(verse);
}

VersePtr VersePtrNew()
{
	return MemPtrNew(sizeof(VerseType));
}

