/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"

static FontID fontNormal;
static FontID fontBold;
static RectangleType renBoldBound;

static void setRenFont(IndexedColorType colorIndex, Boolean bold, Boolean vBoldEnable)
{
	if (os35){
		WinSetTextColor(colorIndex);
	}

	if (vBoldEnable && useVBold){
		vBold = bold;
		FntSetFont(fontNormal);
	} else {
		vBold = false;
		if (bold){
			FntSetFont(fontBold);
		} else {
			FntSetFont(fontNormal);
		}
	}
}

void RenSetBookFont(Boolean vBoldEnable)
{
	return setRenFont(bookColorIndex, boldBook, vBoldEnable);
}

void RenSetChapFont(Boolean vBoldEnable)
{
	return setRenFont(chapterColorIndex, boldChapter, vBoldEnable);
}

void RenSetDescFont(Boolean vBoldEnable)
{
	return setRenFont(descColorIndex, boldDesc, vBoldEnable);
}

void RenSetVerseFont()
{
	return setRenFont(verseColorIndex, boldVerse, false);
}

void RenSetTextFont(Boolean vBoldEnable)
{
	return setRenFont(textColorIndex, boldText, vBoldEnable);
}

void RenOpenRender()
{
	fontNormal = stdFont;
	fontBold = boldFont;
}

void RenCloseRender()
{
}

void RenDrawLine(char* textLine, UInt16 verse, Coord y, Boolean vBoldEnable
	, Boolean rightAligned)
{
	char flag;
	char* verseText;
	UInt16 indentWidth;
	WinHandle boldWin;
	UInt16 error;

	indentWidth = 0;

	flag = textLine[0]; 
	textLine++;
	
	if (flag == 'b'){
		RenSetBookFont(vBoldEnable);
	} else if (flag == 'c') {
		RenSetChapFont(vBoldEnable);
	} else if (flag == 'd') {
		RenSetDescFont(vBoldEnable);
	} else if (flag == 's') {
		indentWidth = textLine[0];
		textLine++;

		RenSetVerseFont();
		verseText = MemPtrNew(8);
		StrIToA(verseText, verse);
		
		if (rightAligned){
			WinDrawChars(verseText, StrLen(verseText), screenWidth - 
				FntCharsWidth(verseText, StrLen(verseText)) - 1, y);
		} else {
			WinDrawChars(verseText, StrLen(verseText), 0, y);
		}
		
		MemPtrFree(verseText);

		RenSetTextFont(vBoldEnable);
	} else {
		RenSetTextFont(vBoldEnable);
	}

	if (rightAligned){
		WinDrawChars(textLine, StrLen(textLine), screenWidth - 
			indentWidth - FntCharsWidth(textLine, StrLen(textLine)), y);
	} else {
		WinDrawChars(textLine, StrLen(textLine), indentWidth, y);
	}

	if (vBold){
		boldWin = WinCreateOffscreenWindow(screenWidth, lineSpace, screenFormat, &error);
		
		renBoldBound.topLeft.y = y;
		renBoldBound.extent.y = lineSpace;

		if (rightAligned){
			renBoldBound.topLeft.x = 0;
			renBoldBound.extent.x = screenWidth - indentWidth;
			WinCopyRectangle(NULL, boldWin, &renBoldBound, 1, 0, winPaint);
			WinCopyRectangle(NULL, boldWin, &renBoldBound, 0, 0, winOverlay);
		} else {
			renBoldBound.topLeft.x = indentWidth;
			renBoldBound.extent.x = screenWidth - indentWidth;
			WinCopyRectangle(NULL, boldWin, &renBoldBound, 0, 0, winPaint);
			WinCopyRectangle(NULL, boldWin, &renBoldBound, 1, 0, winOverlay);
		}
		
		renBoldBound.topLeft.x = 0;
		renBoldBound.topLeft.y = 0;

		if (rightAligned){
			WinCopyRectangle(boldWin, NULL, &renBoldBound, 0, y, winPaint);
		} else {
			WinCopyRectangle(boldWin, NULL, &renBoldBound, indentWidth, y, winPaint);
		}

		WinDeleteWindow(boldWin, false);
	}
}

